# FindSystem match the operating system and set some
# CFLAGS for compile the source code.
#
# The following variables are set:
# CMAKE_C_FLAGS - flags to add to the C compiler for buil arpon
#
# Copyright 2010 <Giuseppe Marco Randazzo> zeld@freaknet.org
#
# Redistribution AND use is allowed according to the terms of the New
# BSD license.

#GET KERNEL VERSION FOR KERNEL 2.4/2.6 PREPROCESSOR MACRO DIRECTIVE
macro(KERNEL_VERSION)
  set(K_VERSION)
  find_program(UNAME NAMES uname PATHS /bin
  /sbin
  /usr/bin
  /usr/sbin
  /usr/local/bin
  /usr/local/sbin)
  if(UNAME)
    execute_process(COMMAND ${UNAME} -r
      OUTPUT_VARIABLE K_VERSION)
    string(REPLACE "." ";" ID_LIST ${K_VERSION})
    list(GET ID_LIST 0 major)
    list(GET ID_LIST 1 minor )
    if(${major}  STREQUAL "2" AND ${minor}  STREQUAL "6")
      message( STATUS "Found Kernel Version 2.6")
      add_definitions( -DLINUX )
    elseif(${major}  STREQUAL "2" AND ${minor}  STREQUAL "4")
      message( STATUS "Found Kernel Version 2.4")
      add_definitions( -D_GNU_SOURCE -D_XOPEN_SOURCE=500 )
    else(${major}  STREQUAL "2" AND ${minor}  STREQUAL "6")
      message( STATUS "Found Kernel Version ${K_VERSION}")
    endif( ${major}  STREQUAL "2" AND ${minor}  STREQUAL "6")
  endif(UNAME)
endmacro(KERNEL_VERSION)

#GET LINUX DISTRIBUTION FOR PREPROCESSOR -DDEBIAN DIRECTIVE USED BY UBUNTU AND DEBIAN
macro(GET_DISTRO)
  set(ID_RELEASE)
  find_program(LSB_RELEASE NAMES lsb_release PATHS /bin
  /sbin
  /usr/bin
  /usr/sbin
  /usr/local/bin
  /usr/local/sbin)
  if(LSB_RELEASE)
    execute_process(COMMAND ${LSB_RELEASE} -i
      OUTPUT_VARIABLE ID_RELEASE)
    string(REGEX REPLACE ".*:" "" ID_RELEASE "${ID_RELEASE}")

    if(${ID_RELEASE} MATCHES "Ubuntu" OR "Debian" )
      message(STATUS "Linux Distro: ${ID_RELEASE}")
      set( UBUNTU TRUE )
      set( DEBIAN TRUE )
      add_definitions( -DDEBIAN )
    endif(${ID_RELEASE} MATCHES "Ubuntu" OR "Debian" )
  endif(LSB_RELEASE)
endmacro(GET_DISTRO)

#Set the header and library name for find process
macro(SET_LIB_HEADER_NAME)
  if(APPLE OR UNIX)
    set(libnet_h libnet.h)
    set(libdnet_h dumbnet.h dnet.h)
    set(libpcap_h pcap.h)
    set(libpthread_h pthread.h)
    set(libnet libnet.a libnet.so libnet.dylib )
    set(libpcap libpcap.a libpcap.so libpcap.dylib)
    set(libpthread libpthread.a libpthread.so libpthread.so.0 libpthread.dylib)
    set(libdnet libdnet.a libdnet.so libdnet.so.1 libdumbnet.a libdumbnet.so libdnet.dylib )
  elseif(WIN32)
    message(FATAL_ERROR "Windows Operating System Not Supported")
  else(APPLE)
    message(FATAL_ERROR "Unknown OS! Unable to detect system library.")
  endif(APPLE OR UNIX)
endmacro(SET_LIB_HEADER_NAME)

if( APPLE )
  # Mac OS X specific code
  message(STATUS "Operating System Found: OSX")
  SET_LIB_HEADER_NAME()
elseif( UNIX )
  if(${CMAKE_SYSTEM_NAME} MATCHES "FreeBSD")
    # FreeBSD specific code
    message(STATUS "Operating System Found: FreeBSD")
    SET_LIB_HEADER_NAME()
  endif(${CMAKE_SYSTEM_NAME} MATCHES "FreeBSD")

  if(${CMAKE_SYSTEM_NAME} MATCHES "NetBSD")
    # NetBSD specific code
    message(STATUS "Operating System Found: NetBSD")
    set(CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -DNETBSD")
    SET_LIB_HEADER_NAME()
  endif(${CMAKE_SYSTEM_NAME} MATCHES "NetBSD")

  if(${CMAKE_SYSTEM_NAME} MATCHES "OpenBSD")
    # OpenBSD specific code
    message(STATUS "Operating System Found: OpenBSD")
    set(CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -DOPENBSD")
    SET_LIB_HEADER_NAME()
  endif(${CMAKE_SYSTEM_NAME} MATCHES "OpenBSD")
  if(${CMAKE_SYSTEM_NAME} MATCHES "Linux")
    # Linux specific code
    message(STATUS "Operating System: GNU/Linux")
    set(CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -DLINUX")
    KERNEL_VERSION()
    GET_DISTRO()
    SET_LIB_HEADER_NAME()
  endif(${CMAKE_SYSTEM_NAME} MATCHES "Linux")
elseif( WIN32 )
  message(FATAL_ERROR "Windows Operating System Not Supported")
else( APPLE )
  message(FATAL_ERROR "Operating System Not Supported")
endif( APPLE )
